@@ -1,10 +1,14 @@
 var classQore_1_1SSH2_1_1SSH2Base =
 [
+    [ "clearStats", "classQore_1_1SSH2_1_1SSH2Base.html#a90249cd534b8afbe72382ff57d4991d8", null ],
+    [ "clearWarningQueue", "classQore_1_1SSH2_1_1SSH2Base.html#a505fbb0d6916dbc59db737fd2aa2c625", null ],
     [ "connect", "classQore_1_1SSH2_1_1SSH2Base.html#abe3efc3098258f858adf317f8f99d48c", null ],
     [ "connected", "classQore_1_1SSH2_1_1SSH2Base.html#a829f2dcc70d888ef25550c28f830b7f3", null ],
     [ "constructor", "classQore_1_1SSH2_1_1SSH2Base.html#a1c7c23112d00c4932adf2235f87eaf28", null ],
     [ "disconnect", "classQore_1_1SSH2_1_1SSH2Base.html#a76365660f99862c84cce8ea6c0c38bdd", null ],
+    [ "getUsageInfo", "classQore_1_1SSH2_1_1SSH2Base.html#a0ec8e9522de48f70d4f79e0db6d71997", null ],
     [ "setKeys", "classQore_1_1SSH2_1_1SSH2Base.html#aeeddcf778158affb5902fa0f92579f55", null ],
     [ "setPassword", "classQore_1_1SSH2_1_1SSH2Base.html#a0b9dec26b401e77a9ec569653653d1aa", null ],
-    [ "setUser", "classQore_1_1SSH2_1_1SSH2Base.html#a2e144927b3312143bd4d972b7a27a213", null ]
+    [ "setUser", "classQore_1_1SSH2_1_1SSH2Base.html#a2e144927b3312143bd4d972b7a27a213", null ],
+    [ "setWarningQueue", "classQore_1_1SSH2_1_1SSH2Base.html#a9fc0a8c391649e24a7e3c504d111f344", null ]
 ];
\ No newline at end of file
