@@ -4,6 +4,7 @@
   SSH2/SFTP integration to QORE
 
   Copyright 2009 Wolfgang Ritzinger
+  Copyright (C) 2010 - 2014 Qore Technologies, sro
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -25,7 +26,7 @@
 #define _QORE_SSH2_MODULE_H
 
 // include configure defines first
-#include "../config.h"
+#include <config.h>
 
 // include Qore API
 #include <qore/Qore.h>
